var bcrypt = require('bcrypt-nodejs');
var crypto = require('crypto');
var BigNumber = require('bignumber.js');
var emptyState = require('../empty_state.js');
var Receipt = require('./receipt.js');
var mongoose = require('../mongoose.js')();
var moment = require('moment');

var userSchema = new mongoose.Schema({
  email: { type: String, unique: true, lowercase: true },
  fromEmail: { type: String, lowercase: true },
  password: String,
  apiKey: String,
  apiSecret: String,
  silentMode: { type: Boolean, default: false },
  tokens: Array,

  profile: {
    firstName: { type: String, default: '' },
    lastName: { type: String, default: '' },
    name: { type: String, default: '' },
    location: { type: String, default: '' },
    website: { type: String, default: '' },
    picture: { type: String, default: '' }
  },

  resetPasswordToken: String,
  resetPasswordExpires: Date,

  created: { type: Date, default: Date.now },
  trialEndDate: Date,
  firstPaymentDate: Date,
  nextPaymentDate: Date,
});

userSchema.pre('save', function (next) {
  if (this.isNew) {
    this.trialEndDate = moment().add(14, 'days').toDate();
    emptyState(this, next);
  } else {
    next();
  }
});

/**
 * Password hash middleware.
 */
userSchema.pre('save', function(next) {
  var user = this;
  if (!user.isModified('password')) return next();
  bcrypt.genSalt(5, function(err, salt) {
    if (err) return next(err);
    bcrypt.hash(user.password, salt, null, function(err, hash) {
      if (err) return next(err);
      user.password = hash;
      next();
    });
  });
});

userSchema.pre('save', function(next) {
  if (this.isNew) {
    this.apiKey = crypto.randomBytes(4).toString('hex');
    this.apiSecret = crypto.randomBytes(16).toString('hex');
  }
  next();
});

/**
 * Helper method for validationg user's password.
 */
userSchema.methods.comparePassword = function(candidatePassword, cb) {
  bcrypt.compare(candidatePassword, this.password, function(err, isMatch) {
    if (err) return cb(err);
    cb(null, isMatch);
  });
};

/**
 * Helper method for getting user's gravatar.
 */
userSchema.methods.gravatar = function(size) {
  if (!size) size = 200;
  if (!this.email) return 'https://gravatar.com/avatar/?s=' + size + '&d=retro';
  var md5 = crypto.createHash('md5').update(this.email).digest('hex');
  return 'https://gravatar.com/avatar/' + md5 + '?s=' + size + '&d=retro';
};

userSchema.methods.balance = function (callback) {
  Receipt
    .find({userId: this._id}, {amount: 1, _id: 0})
    .exec(function (error, results) {
        if (error) return callback(error);
        var totalBalance = new BigNumber('0');
        results.forEach(function (receipt) {
            totalBalance = totalBalance.plus(receipt.amount);
        });
        callback(null, totalBalance.toString());
    });
};

userSchema.methods.receipts = function (limit, callback) {
    Receipt
        .find({userId: this._id})
        .sort({date: -1})
        .limit(limit)
        .exec(callback);
};

userSchema.methods.addPayment = function (amount, comment, data, callback) {
    new Receipt({
        userId: this.id,
        amount: amount,
        comment: comment,
        data: data
    }).save(callback);
};

userSchema.method('toJSON', function() {
  var user = this.toObject();
  delete user.tokens;
  delete user.password;
  delete user.__v;
  user.gravatar = this.gravatar();
  return user;
});

module.exports = mongoose.model('User', userSchema);
